function [chain_state_current,ts_new,dint1t] = binding_folding(chain_state_current,Fc,dint1t,K_2_1_f_f)
%% This function calculates:
% 1. the binding/unbinding rates, based on the model of Chan and Odde (Science, 2008)
% 2. the folding/unfolding rates of the Talin rod based on Yao et al (Nat Commun)

global kont1 kof1 konv nc mr intadd dint1 ion Fb koff_catch
global Rod_contour_length Rod_K0_unfold Rod_K0_refold Rod_D_unfold Rod_D_refold

n_rods = size(chain_state_current,2)-1; % -1 because in the last column I have an indicator of free/bound
   
k  = zeros(nc,1); % koff
k1 = zeros(nc,1); % kon

indbound1  = find(chain_state_current(:,end) == 1); % indeces of bound binders
indunbound = find(chain_state_current(:,end) == 0); % indeces of free binders

koff1 = koffcb(Fc,ion,kof1,Fb,koff_catch); % koff in each clutch
k(indbound1)   = koff1(indbound1);
k1(indunbound) = kont1*dint1t;

K_1_2_f = zeros(nc,length(Rod_K0_refold));
K_2_1_f = zeros(nc,length(Rod_K0_refold));

for i1 = 1:nc
    K_1_2_f(i1,:) = (k_f_bells(Fc(i1)./4.1,Rod_K0_unfold,Rod_D_unfold)); % unfolding rate depending on force (first argument=force already divided by kBT)
    K_2_1_f(i1,:) = abs(K_2_1_f_f(double(Fc(i1)))); % folding rate depending on force
end

kuf = zeros(nc,length(Rod_K0_unfold)); % unfolding rate of rods in bound clutches
kf  = zeros(nc,length(Rod_K0_unfold)); % folding rate of rods in bound clutches
kv  = zeros(nc,1);

% [ indf_row, indf_col ]    = find(chain_state_current(:,1:end-1) == 1  & chain_state_current(:,end) == 1);

[ indf_row, indf_col ]    = find(chain_state_current(:,1:end-1) == 1);
[ indunf_row, indunf_col ]= find(chain_state_current(:,1:end-1) == 0);
  indv                    = find(chain_state_current(:,3)       == 0);
% ATTENTION!! indv indicates binders with domain R3 unfolded, so that vinculin can bind
  

for i1=1:length(indf_row)
    kuf(indf_row(i1),indf_col(i1))  =...
        K_1_2_f (indf_row(i1),indf_col(i1)); % only for folded domains I compute the unfolding rate
end

kuf(indunbound,:) = 0; % to avoid unfolding of domains in free binders
for i1=1:length(indunf_row)
    kf(indunf_row(i1), indunf_col(i1))=...
        K_2_1_f (indunf_row(i1), indunf_col(i1)); % only for unfolded domains I comupute the folding rate
end

for i1=1:length(indv) %
    kv(indv(i1)) = konv; % for binders with domain R3 unfolded, I put the on-rate of vinculin
end



% 1.--------- IF ALL CLUTCHES ARE FREE --------------------------------------

if isempty(indbound1) 
    
    teventub = inf;                 % unbinding is impossible if they are all free
    teventf = inf*ones(nc,n_rods);  % folding is impossible because they are already all folded
    teventuf = inf*ones(nc,n_rods); % unfolding is impossible because they are all free   
    teventvinc = inf*ones(nc,1);    % vinculin binding is impossible because they are all free and folded

% 2.--------- IF WE HAVE SOME BOUND CLUTCHES ---------------------------------

else                 
    teventub = -log(rand(nc,1))./k;  % for the bound, compute unbinding time
    % It looks like I compute unbinding time for all binders, but k is
    % the koff and is =0 for the free binders, so teventub = inf ---> perfect

    if isempty(indunf_row) % 2.1 ---------- IF ALL CLUTCHES AND DOMAINS ARE FOLDED ---------------------------
        % (it means if all bound clutches are folded, because the free are always folded)
        
        teventf    = inf*ones(nc,n_rods); % refolding is impossible because they are already folded           
        teventvinc = inf*ones(nc,1);      % vinculin binding is impossible because they are all folded
        
    else                   % 2.2 ---------- IF SOME CLUTCHES AND DOMAINS ARE UNFOLDED ------------------------
               
        teventf    = -log(rand(nc,n_rods))./kf; % refolding time for the unfolded clutches
        
        teventvinc = -log(rand(nc,1))./(kv); % time for vinculin binding
    end
    
    if isempty(indf_row)   %  2.3 ---------- IF ALL CLUTCHES AND DOMAINS ARE UNFOLDED -----------
    
        teventuf = inf*ones(nc,n_rods);      % unfolding time for the folded domains  
        teventvinc = -log(rand(nc,1))./(kv); % time for vinculin binding      CHIARA
    
    else                 %  2.4 ---------- IF SOME CLUTCHES AND DOMAINS ARE FOLDED -----------------
        
        teventuf = -log(rand(nc,n_rods))./kuf; % unfolding time for folded domains
        teventvinc = -log(rand(nc,1))./(kv); % time for vinculin binding      CHIARA       
    end
end

% 3.-------------- IF ALL CLUTCHES ARE BOUNDED --------------------------  

if isempty(indunbound)                                       
    teventb1 = inf; % binding is impossible      

% 4.-------------- IF WE HAVE SOME FREE CLUTCHES ---------------------------------
else                                                   
    teventb1 = -log(rand(nc,1))./k1; % binding time for free clutches                              
end 

state_before = chain_state_current;


%% We find which event happen first and execute IT:
[dtf, indf]       = min(teventf,[],1) ;  % teventf = 1200*11, I take the minimum of each column:
                                         % for each of the 11 talin domains, 
                                         % I look in which of the 1200 binders it folds first
[dtuf, induf]     = min(teventuf,[],1) ;  
[dtvinc, indvinc] = min(teventvinc) ;
[dtb1, indb1]     = min(teventb1);
[dtub, indub]     = min(teventub) ;


% find global min 
[dtglobal,indglobal]= min([dtf dtuf dtvinc dtub dtb1]); % the last 2 are inverted wrt before          
% I find the minimun of the minimum, so the event happening first, and I execute only that one      


if (indglobal==n_rods*2+3)    % binding of unbound clutches
    ts_new = dtb1;
    chain_state_current(indb1,end) = 1;    % I put it bound
end

if (indglobal==n_rods*2+2)   % unbinding of bound clutches
    ts_new = dtub;
    chain_state_current(indub,end)     = 0;  % I put it free
    chain_state_current(indub,1:end-1) = 1;  % I put it all the 11 domains folded

    %% VINCULIN changes integrin density 

    % For the binders that pass from bound to free

    % and before had vinculin bound, I increase integrin density
    dint1t = dint1t + intadd.*length(find(state_before(indub,3)+ state_before(indub,end).*10 == 12));

    % and before had domain R3 folded, I decrease integrin density
    dint1t = dint1t - intadd.*length(find(state_before(indub,3)+ state_before(indub,end).*10 == 10));

    % and before had domain R3 unfolded, but without vinculin, I decrease integrin density
    dint1t = dint1t - intadd.*length(find(state_before(indub,3)+ state_before(indub,end).*10 == 11));

    if dint1t < dint1
        dint1t = dint1;

    elseif dint1t > mr
        dint1t = mr;
    end
end

if (indglobal==n_rods*2+1)       % vinculin binding of unfolded R3
    ts_new = dtvinc;
    chain_state_current(indvinc,3) = 2;  
end

if (indglobal<=n_rods)            % folding of unfolded clutches
    [ts_new ,ind_aux] = min(dtf); 
    % ind_aux = number of the domain (among the 11) where the event happens, so column where I must update
    % indf = vector of 11 components saying for each of the 11 domains, the number of the binder (among the nc) 
    % in which the domain folds first, so indf(ind_aux) gives me the line where I must update
    chain_state_current(indf(ind_aux), ind_aux) = 1;
end

if (indglobal>=n_rods+1) && (indglobal<=n_rods*2) % unfolding of folded clutches
    [ts_new ,ind_aux] = min(dtuf);
    chain_state_current(induf(ind_aux), ind_aux) = 0; 
end

end


