function [lines_out, closest]=data_myproblem(lines,focals,excl)

%data.m reads in, plots and modifies original data
%INPUT:   lines contains position of points to compute displacement x1 y1
%         focals contains positions of focals in form x1 y1
%OUTPUT:  lines_out contains displacement data in form x1 y1

% remove all displacements which are closer to any adhesion point than delta1
% This is important since we assume point-like forces which diverge at the origin;

delta1 = excl;

Nfocals = size(focals,1);
Nlines = size(lines,1);

fprintf('Cleaning process initiated\n')
fprintf('%d displacement points \n',Nlines)
fprintf('%d force points\n',Nfocals)

% plot displacements
plot(lines(:,1),lines(:,2),'b.'); hold on;
%splot(focals,'ks')

% check if:
% 1. any displacements are too close to focals for use Green function 
% 2. identidied displ points closest to focal points 
% and if so, delete them and send lines_out out
dellines = zeros(Nlines,1);
dellines_perForcePoint = zeros(Nfocals,Nlines);
maxdispl = 0;
maxindex = 1;

for i = 1:Nfocals
r1 = focals(i,1:2);
    for j = 1:Nlines
        r2 = lines(j,1:2);
        dr = r2-r1;
        l(i,j) = sqrt(dr*dr');
        if (l(i,j) < delta1)
            dellines(j) = 1;
            dellines_perForcePoint(i,j) = 1;
%            splot(lines(j,:),'ks')
        end 
    end
end
delind = find(dellines);
lines(delind,:) = [];
l(:,delind)=[];
Nlines = size(lines,1);

for i = 1:Nfocals
    [aa,bb] = sort(l(i,:));
    closest(i) = bb(1,1);
end
lines_out=lines;

fprintf('Minimum distance focals - displacements required %d pixels\n',delta1)
fprintf('%d displacement points deleted\n',size(delind,1))
fprintf('%d displacements points left\n',Nlines)
fprintf('Ended! Points too close to force points removed \n')

