function G = greens_Sw_X(r)
%GREENS Greens function for traction on an elastic halfspace.
% GREENS(r), where r is a 2d vector, returns the Greens function (a 2x2 matrix) for
% tangential traction on an elastic halfspace. We use E = 1 and sigma = 1/2.
%
% Ulrich Schwarz, MPI Colloids and Interfaces, Germany

if (nargin ~= 1)
   error('Only one input variable allowed')
end

[n,m] = size(r);
if (~((n == 1 & m == 2) | (m == 1 & n == 2))) 
   error('Input must be a 2d vector')
end

x = r(1);
y = r(2);
mag = sqrt(x*x+y*y);

if (mag == 0)
   error('Vector has zero length')
end

G(1,1) = 3/(4*pi*mag)*(1+x*x/mag/mag);


