clear all
close all
clc

%% Clutch model with talin (12 domains)
% Clutch model taken from Elosegui-Artola et al., Nature Cell Biology 18(2016) 540.
% Talin model taken from Yao et al., Nature Communications 7, (2016) 11966.
% Green's functions for the substrate
% Gillespie (variable timestep)
% pN nm

% xtal= end-to-end length of talin rod (neck + folded domains + unfolded domains),that is everything apart from the head (offset).
% endend_tal_MC = offset + xtal

addpath('./functions');
warning('off')

MC = 10; % Number of MonteCarlo simulations                
               
t_10MC           = cell(MC,1); % only one column
xtot_10MC        = cell(MC,2);
xtal_10MC        = cell(MC,2);
xtal_max_10MC    = cell(MC,2);
endend_tal_10MC  = cell(MC,2);
xint_10MC        = cell(MC,2);
xint_max_10MC    = cell(MC,2);
xs_10MC          = cell(MC,2);
Fc_time_nc_10MC  = cell(MC,2);
Fc_10MC          = cell(MC,2);
Fc_max_10MC      = cell(MC,2);
v_10MC           = cell(MC,2);
chain_state_10MC = cell(MC,2);
dint1_norm_10MC  = cell(MC,2);
nb1_10MC         = cell(MC,2);
nub_10MC         = cell(MC,2);
%nf_10MC         = cell(MC,2);
%nuf_10MC        = cell(MC,2);
nv_10MC          = cell(MC,2);
Fc_min_10MC      = cell(MC,2);
Pb_10MC          = cell(MC,2);
P_10MC           = cell(MC,2);


for mc = 1 : MC
    
    cunreal=0;
    out = false;

    tf     = 30;   % Final time (s)
    nsteps = 20;   % number of timesteps after which variables.mat is saved
    dt_max = 0.1;   % maximum timestep 

    parameters_clutch    % Load parameters associated with the molecular clutch
    parameters_talin;    % Load parameters associated with Talin rod:  Rod_contour_length 
                                                                    %  Rod_K0_unfold
                                                                    %  Rod_K0_refold
                                                                    %  Rod_D_unfold
                                                                    %  Rod_D_refold 
    load K_2_1_f_f.mat   % Load a @function K_2_1 for folding rates depending on force 
                         % (computed in K_folding.m, which uses talin_domain_kf_f1.m)                 

    % Variables during time, concatenating each big step made of nsteps

    t_MC           = []; % time
    xtot_MC        = []; % total delta x (substrate+integrin+talin), mean over bound binders
    xtal_MC        = []; % delta x talin, mean over bound binders
    xtal_max_MC    = []; 
    endend_tal_MC  = []; 
    xint_MC        = []; % delta x integrin, mean over bound binders
    xint_max_MC    = [];
    xs_MC          = []; % delta x substrate
    Fc_MC          = []; % force on integrin, mean over bound binders
    Fc_max_MC      = [];
    Fc_time_nc_MC  = []; % force on integrin, for all binders, during time
    chain_state_MC = []; % mean state of the binders (for folded/unfolded state the average is only over bound binders)
    v_MC           = []; % actin velocity 
    nb1_MC         = []; % number of bound binders (nc = nb1 + nub)
    nub_MC         = []; % number of free binders
    nf_MC          = []; % for each of the 11 talin domains, number of bound binders in which it is folded
    nuf_MC         = []; % for each of the 11 talin domains, number of bound binders in which it is unfolded
    nv_MC          = []; % number of binders with vinculin bound
    dint1_norm_MC  = []; % integrin density normalized with the initial density
    Fc_min_MC       = [];
    
    % Initialize variables used for each big step

    xtotal      = zeros(nsteps, nc);   
    xtot_mean   = zeros(nsteps, 1);  
    xtalin      = zeros(nsteps, nc); 
    xtal_max    = zeros(nsteps, 1);  
    xtal_mean   = zeros(nsteps, 1);
    xintegrin   = zeros(nsteps, nc); 
    xint_max    = zeros(nsteps, 1);  
    xint_mean   = zeros(nsteps, 1); 
    xsub        = zeros(nsteps, nc); 
    xsub_mean   = zeros(nsteps, 1); 
    Fclutch     = zeros(nsteps, nc); 
    Fc_max      = zeros(nsteps, 1); 
    Fc_mean     = zeros(nsteps, 1); 
    states_mean = zeros(nsteps, length(Rod_contour_length)+1);    
    v           = zeros(nsteps, 1); 
    nb1         = zeros(nsteps, 1);
    nub         = zeros(nsteps, 1);
    nf          = nc * ones(nsteps, length(Rod_contour_length));
    nuf         = zeros(nsteps, length(Rod_contour_length));
    nv          = zeros(nsteps, 1); 
    dint1t      = zeros(nsteps, 1);
    t           = [];

    % Initialize at t=0 the variables for each big step
    Fc_min          = zeros(nsteps, 1);
    Fc_min(1)       = 0;
    xtot           = zeros(nc,1);
    xtotal(1,:)    = xtot';     
    xtal           = zeros(nc,1);
    xtalin(1,:)    = xtal';
    xint           = zeros(nc,1);
    xintegrin(1,:) = xint';
    xs             = zeros(nc,1);
    xsub(1, :)     = xs' ;  
    Fc             = zeros(nc,1);
    Fclutch(1,:)   = Fc';       
    v(1)           = vu;  
    
    % Folding status of each clutch (0 unfolded | 1 folded | 2 vinculin-bound) [folding as in Yao, the contrary of Pere]
    % Binding status of each clutch (0 unbound  | 1 integrin )
    % chain_state_start: nc rows * 12 columns (folded/unfolded state for each of the 11 domains + free/bound state)
    % All talin domains start folded, and all binders start free
    chain_state_start        = ones(nc,length(Rod_K0_unfold)+1);   
    chain_state_start(:,end) = 0;         
    chain_state_current = chain_state_start;        
    chain_state(1,:,:)  = chain_state_current';    % matrix saving in time (1st dimension)
                                                   % the state of the chain (2nd dimension)
                                                   % for all nc binders (3rd dimension)
    dint1t(1)      = dint1; 
    t(1)           = 0; 

    %% GILLESPIE loop

    taux = t(1); % needed at the end of each big step, in case the simulation stops before reaching tf
    csteps = 0;  % big steps

    while taux <= tf % Loop until the final time, in case it goes out because c1 < nsteps is not satisfied
          csteps = csteps + 1; 
          c1 = 1;              % small steps

          while c1 < nsteps && taux <= tf
                c1 = c1+1;

                state_temp = chain_state_current;
                Fc_temp = Fc;
                dint1_temp = dint1t(c1-1);
                xtot_temp = xtot;
                [chain_state_current,ts,dint1t_aux] = binding_folding(chain_state_current,Fc,dint1t(c1-1),K_2_1_f_f);

                % If the spontaneous timestep is too big, in order to see the evolution in time of the variables
                % we update the time with a fixed timestep dt_max, without changing the chain state
                % (so that the changing of state is kept spontaneous)
                if ts > dt_max
                   ts = dt_max;
                   chain_state_current = state_temp;
                   dint1t_aux = dint1_temp;
                end

                boundbin = logical(chain_state_current(:,end)); % Only needed if there is a second integrin type 2, 
                                                                % so that number 2 is converted into number 1
                vf = vu.*(1 - sum(Fc)/Fstall);

                if vf<0
                   sprintf('negative velocity')
                end

                xtot = xtot + vf.*ts.*boundbin;
                xtot_bound = xtot(boundbin);
                F_bound = Fc(boundbin);

                % If there are bound binders, I do the balance of forces, otherwise the fn returns empty instead of zero
                if sum(chain_state_current(:,end)==1)>0 
                    [Fc_bound, out] = Fc_vs_contour_states(xtot_bound, F_bound, chain_state_current, kc, G_Sw_X, Rod_contour_length, t,c1,ts, Fstall);

                    while out==true
                        cunreal=cunreal+1;

                        ts = ts/2;
                        chain_state_current = state_temp;
                        Fc = Fc_temp;
                        dint1t_aux = dint1_temp;

                        boundbin = logical(chain_state_current(:,end)); 
                        
                        vf = vu.*(1 - sum(Fc)/Fstall);

                        xtot = xtot_temp + vf.*ts.*boundbin;
                        xtot_bound = xtot(boundbin);
                        F_bound = Fc(boundbin);

                        % If there are bound binders, I do the balance of forces, otherwise the fn returns empty instead of zero
                        if sum(chain_state_current(:,end)==1)>0 
                           [Fc_bound, out] = Fc_vs_contour_states(xtot_bound, F_bound, chain_state_current, kc, G_Sw_X, Rod_contour_length,t,c1,ts, Fstall);
                        else
                           out = false; 
                        end 
                    end
                    %  end of the while

                    Fc(boundbin==1) = Fc_bound; 
                    Fc(boundbin==0) = 0;
                else
                    Fc = zeros(nc,1);
                end   

                xint = Fc./kc;     % vector, one value for each binder
                xs = G_Sw_X * Fc;  % vector, one value for each binder, free included

                % Free binders only have the displacement of the substrate
                xtot(boundbin == 0) = xs(boundbin == 0);   % I need the xs computed after balance of forces   
                xint(boundbin == 0) = 0;                   % not needed, because it is computed with F=0
                xtal = xtot - xint - xs;
                xtal(boundbin == 0) = 0; 

                if min(xtal)<0
                   min(xtal) 
                   sprintf('xtalin negative')
                end

                % size-increasing vectors (so costly! we can do it with cat..)
                t          (c1)     = t(c1-1) + ts;        
                xtotal     (c1,:)   = xtot';                
                xtalin     (c1,:)   = xtal';
                xintegrin  (c1,:)   = xint'; 
                Fclutch    (c1,:)   = Fc';  
                xsub       (c1,:)   = xs';                                 
                v          (c1)     = vf';                  
                chain_state(c1,:,:) = chain_state_current'; 
                nb1        (c1)     = sum(chain_state_current(:,end)== 1); % chain_state_current is nc * 12
                nub        (c1)     = sum(chain_state_current(:,end)== 0); 
                nf         (c1,:)   = sum(chain_state_current(boundbin,1:end-1) == 1,1); % sum elements of the same column
                nuf        (c1,:)   = sum(chain_state_current(boundbin,1:end-1) == 0,1); % sum elements of the same column  
                nv         (c1)     = sum(chain_state_current(boundbin,3) == 2,1);       % sum elements of the same column
                dint1t     (c1)     = dint1t_aux;

                taux = t(c1); % time reached, needed in the next timestep

          end

            % I take exactly the size of the big step in time I did
            xtotal      = xtotal(1:length(t),:);    
            xtalin      = xtalin(1:length(t),:); 
            xintegrin   = xintegrin(1:length(t),:);
            xsub        = xsub(1:length(t), :);   
            chain_state = chain_state(1:length(t),:,:); 
            Fclutch     = Fclutch(1:length(t),:);    
            v           = v(1:length(t)); 
            dint1t      = dint1t(1:length(t)); 
            nb1         = nb1(1:length(t)); 
            nub         = nub(1:length(t)); 
            nf          = nf(1:length(t),:); 
            nuf         = nuf(1:length(t),:); 
            nv          = nv(1:length(t));

            % Computing average over binders, max over binders.... for xtot,xtal,xint,xsub,Fc
            temp = chain_state; % matrix saving in time (righe) for all the 11 domains+bound/free state 
                                % (12 columns), the state of the binders
            Fclutch_temp = Fclutch;
            xsub_mean = mean(xsub,2);

            % MEAN OVER BOUND BINDERS ONLY
            for time = 1:length(t)
                for binder = 1:nc
                    if temp(time,end,binder)==0 % for free binders I put NaN in the following variables
                        xtotal(time,binder) = NaN;
                        xtalin(time,binder) = NaN;
                        xintegrin(time,binder) = NaN;
                        Fclutch_temp(time,binder) = NaN;
                    end
                end
            end

            % xtalin has size nsteps*nc 
            xtal_max = max(xtalin,[],2); 
            xint_max = max(xintegrin,[],2);
            Fc_max   = max(Fclutch_temp,[],2);
            Fc_min = min(Fclutch_temp,[],2);

            % mean (a,2) does the average over elements of the same row, so its mean over binders (gives me a column vector)
            xtot_mean = mean(xtotal,2,'omitnan'); 
            xtal_mean = mean(xtalin,2,'omitnan');
            xint_mean = mean(xintegrin,2,'omitnan');
            Fc_mean   = mean(Fclutch_temp,2,'omitnan');

            % In timesteps where all binders are free, instead of NaN we put zeros,
            % so the average in time (done in the postprocessing) is done with zeros
            xtal_max(isnan(xtal_max))   = 0;
            xint_max(isnan(xint_max))   = 0;
            Fc_max(isnan(Fc_max))       = 0; 
            xtot_mean(isnan(xtot_mean)) = 0;
            xtal_mean(isnan(xtal_mean)) = 0;
            xint_mean(isnan(xint_mean)) = 0;
            Fc_mean(isnan(Fc_mean))     = 0;
            Fc_min(isnan(Fc_min))       = 0; 

            %% Computing average over binders for chain_state

            % we transform numbers 2 indicating vinculin bound, into numbers 0, so
            % that in the mean they are considered as unfolded
            temp(temp==2) = 0;

            % MEAN OVER ALL BINDERS
            % states_mean = mean(temp,3); % doing average also on the last column, but its fine we dont use it

            % MEAN OVER BOUND BINDERS ONLY
            % for each time, for free binders, we put NaN in the 11 positions
            for time = 1:length(t)
                for binder = 1: nc
                    if temp(time,end,binder)==0
                       temp(time,1:11,binder) = NaN;
                    end
                end
            end
            states_mean = mean(temp,3,'omitnan'); 

            % concatenating the big steps for all variables 
            t_MC           = [t_MC; t'];
            xtal_max_MC    = [xtal_max_MC; xtal_max];
            xint_max_MC    = [xint_max_MC; xint_max];
            Fc_max_MC      = [Fc_max_MC; Fc_max];
            xtot_MC        = [xtot_MC; xtot_mean];
            xtal_MC        = [xtal_MC; xtal_mean];
            xint_MC        = [xint_MC; xint_mean];
            xs_MC          = [xs_MC; xsub_mean];
            Fc_MC          = [Fc_MC; Fc_mean];
            Fc_time_nc_MC  = [Fc_time_nc_MC; Fclutch];
            v_MC           = [v_MC; v];
            chain_state_MC = [chain_state_MC; states_mean];
            dint1_norm_MC  = [dint1_norm_MC; dint1t./dint1];
            nb1_MC         = [nb1_MC; nb1];
            nub_MC         = [nub_MC; nub];
            nf_MC          = [nf_MC; nf];
            nuf_MC         = [nuf_MC; nuf];
            nv_MC          = [nv_MC; nv];

            Fc_min_MC       = [Fc_min_MC; Fc_min];

            % In order to start next big step with the time reached, and all the actual variables 
            t = [];
            t          (1)     = taux; 
            dint1t     (1)     = dint1t(end);
            xtotal     (1,:)   = xtotal(end,:); 
            xtalin     (1,:)   = xtalin(end,:); 
            xintegrin  (1,:)   = xintegrin(end,:); 
            xsub       (1,:)   = xsub(end, :); 
            Fclutch    (1,:)   = Fclutch(end,:);  
            v          (1)     = v(end); 
            chain_state(1,:,:) = chain_state(end,:,:);
            nb1        (1)     = nb1(end);       
            nub        (1)     = nub(end);               
            nf         (1,:)   = nf(end,:);      
            nuf        (1,:)   = nuf(end,:);   
            nv         (1)     = nv(end); 

            % In order to put endend_tal_MC = 0 (and not =offset=10 nm) when all
            % binders are free (xtal_MC that is xtalin mean over bound binders is zero)
            temp = zeros(length(xtal_MC),1);
            temp(xtal_MC~=0) = offset;
            endend_tal_MC = temp + xtal_MC;
                        
            Pb_MC = nb1_MC./nc;
            P_MC = sum(Fc_time_nc_MC,2)./(pi*a^2); 
            P_MC = P_MC.*1e6; % to pass from MPa to Pa
            
            if mc==1
                % save variables in the file .mat
                system('rm variables_1MC.mat');
                save('variables_1MC.mat','t_MC','xtal_max_MC','xint_max_MC','Fc_max_MC',...
                    'xtot_MC','xtal_MC','endend_tal_MC','xint_MC','xs_MC','Fc_MC','Fc_time_nc_MC','v_MC',...
                    'chain_state_MC','dint1_norm_MC','nb1_MC','nub_MC','nf_MC','nuf_MC','nv_MC', 'E', 'Fc_min_MC','Pb_MC','P_MC'); 
            end
            
            if min(Fc_min_MC)<0
                sprintf('Fc_min negative')
                [val,idx]=min(Fc_min_MC);
                t_MC(idx)
            end

    end
 
%% average over MC
    
    t_10MC           {mc,1} = t_MC;          
    xtot_10MC        {mc,1} = xtot_MC;
    xtal_10MC        {mc,1} = xtal_MC;
    xtal_max_10MC    {mc,1} = xtal_max_MC;  
    endend_tal_10MC  {mc,1} = endend_tal_MC;
    xint_10MC        {mc,1} = xint_MC;
    xint_max_10MC    {mc,1} = xint_max_MC;
    xs_10MC          {mc,1} = xs_MC;
    Fc_time_nc_10MC  {mc,1} = Fc_time_nc_MC;
    Fc_10MC          {mc,1} = Fc_MC;
    Fc_max_10MC      {mc,1} = Fc_max_MC;
    v_10MC           {mc,1} = v_MC;
    chain_state_10MC {mc,1} = chain_state_MC;
    dint1_norm_10MC  {mc,1} = dint1_norm_MC;
    nb1_10MC         {mc,1} = nb1_MC;
    nub_10MC         {mc,1} = nub_MC;
    %nf_10MC         {mc,1} = nf_MC;
    %nuf_10MC        {mc,1} = nuf_MC;
    nv_10MC          {mc,1} = nv_MC;
    Fc_min_10MC      {mc,1} = Fc_min_MC;
    Pb_10MC          {mc,1} = Pb_MC;
    P_10MC           {mc,1} = P_MC;
    
    % In the first column of the cells nb1_10MC I have saved the vector with
    % the values of the variable nb1_MC during time, for each MC simulation.
    % Now in the second column I save the WEIGHTED AVERAGE of the variable
    % during time, because the timesteps have different lenghts
    
    timestep = diff(t_MC);
    
    xtot_10MC       {mc,2} = sum((xtot_MC(1:end-1).* timestep)/t_MC(end));  
    xtal_10MC       {mc,2} = sum((xtal_MC(1:end-1).* timestep)/t_MC(end));  
    xtal_max_10MC   {mc,2} = sum((xtal_max_MC(1:end-1).* timestep)/t_MC(end));  
    endend_tal_10MC {mc,2} = sum((endend_tal_MC(1:end-1).* timestep)/t_MC(end));  
    xint_10MC       {mc,2} = sum((xint_MC(1:end-1).* timestep)/t_MC(end));  
    xint_max_10MC   {mc,2} = sum((xint_max_MC(1:end-1).* timestep)/t_MC(end));  
    xs_10MC         {mc,2} = sum((xs_MC(1:end-1).* timestep)/t_MC(end)); 
    Fc_10MC         {mc,2} = sum((Fc_MC(1:end-1).* timestep)/t_MC(end));  
    Fc_max_10MC     {mc,2} = sum((Fc_max_MC(1:end-1).* timestep)/t_MC(end)); 
    v_10MC          {mc,2} = sum((v_MC(1:end-1).* timestep)/t_MC(end));  
    dint1_norm_10MC {mc,2} = sum((dint1_norm_MC(1:end-1).* timestep)/t_MC(end));  
    nb1_10MC        {mc,2} = sum((nb1_MC(1:end-1).* timestep)/t_MC(end));  
    nub_10MC        {mc,2} = sum((nub_MC(1:end-1).* timestep)/t_MC(end));  
    nv_10MC         {mc,2} = sum((nv_MC(1:end-1).* timestep)/t_MC(end));  
    Fc_min_10MC     {mc,2} = sum((Fc_min_MC(1:end-1).* timestep)/t_MC(end));  
    Pb_10MC         {mc,2} = sum((Pb_MC(1:end-1).* timestep)/t_MC(end)); 
    P_10MC          {mc,2}  = sum((P_MC(1:end-1).* timestep)/t_MC(end));
 
end


system('rm variables_10MC.mat');
save('variables_10MC.mat','t_10MC','xtot_10MC','xtal_10MC','xtal_max_10MC','endend_tal_10MC','xint_10MC','xint_max_10MC','xs_10MC', ...
                     'Fc_time_nc_10MC','Fc_10MC','Fc_max_10MC','v_10MC','chain_state_10MC',...
                     'dint1_norm_10MC','nb1_10MC','nub_10MC','nv_10MC','Fc_min_10MC','Pb_10MC','P_10MC','E');
                
